from savant_rs.primitives import Object, Value, Attribute, ParentObject, BBox
from timeit import default_timer as timer

o = Object(
    id=1,
    creator="some",
    label="person",
    bbox=BBox(0.1, 0.2, 0.3, 0.4),
    confidence=0.5,
    attributes={},
    parent=ParentObject(id=2, creator="some", label="car"),
    track_id=None,
)


t = timer()

bts = bytes(256)

a = Attribute(creator="other", name="attr", values=[
    # Value.bytes(dims=[8, 3, 8, 8], blob=bts, confidence=None),
    Value.integer(1, confidence=0.5),
    Value.integer(2, confidence=0.5),
    Value.integer(3, confidence=0.5),
    Value.integer(4, confidence=0.5),
])

for _ in range(1_000):
    o.set_attribute(a)
    a = o.get_attribute(creator="other", name="attr")
    # x = a.name

print(timer() - t)




"""Utility for scale rotated bboxes."""
import numpy as np
from savant_rs.primitives import BBox


def scale_rbbox(
    bboxes: np.ndarray, scale_factor_x: float, scale_factor_y: float
) -> np.ndarray:
    """Scaling rotated boxes.

    :param bboxes: np array of bboxes, shape Nx5. Row is [cx, cy, w, h, angle]
    :param scale_factor_x: scale factor for x coordinates
    :param scale_factor_y: scale factor for y coordinates
    """
    bboxes_zero_angle = bboxes[np.mod(bboxes[:, 4], 90) == 0]
    bboxes_not_zero_angle = bboxes[np.mod(bboxes[:, 4], 90) != 0]

    if bboxes_not_zero_angle.shape[0] > 0:
        scale_x = np.array([scale_factor_x] * bboxes_not_zero_angle.shape[0])
        scale_y = np.array([scale_factor_y] * bboxes_not_zero_angle.shape[0])
        scale_x_2 = scale_x * scale_x
        scale_y_2 = scale_y * scale_y
        cotan = 1 / np.tan(bboxes_not_zero_angle[:, 4] / 180 * np.pi)
        cotan_2 = cotan * cotan
        scale_angle = np.arccos(
            scale_x
            * np.sign(bboxes_not_zero_angle[:, 4])
            / np.sqrt(scale_x_2 + scale_y_2 * cotan * cotan)
        )
        nscale_height = np.sqrt(scale_x_2 + scale_y_2 * cotan_2) / np.sqrt(1 + cotan_2)
        ayh = 1 / np.tan((90 - bboxes_not_zero_angle[:, 4]) / 180 * np.pi)
        nscale_width = np.sqrt(scale_x_2 + scale_y_2 * ayh * ayh) / np.sqrt(
            1 + ayh * ayh
        )
        bboxes_not_zero_angle[:, 4] = 90 - (scale_angle * 180) / np.pi
        bboxes_not_zero_angle[:, 3] = bboxes_not_zero_angle[:, 3] * nscale_height
        bboxes_not_zero_angle[:, 2] = bboxes_not_zero_angle[:, 2] * nscale_width
        bboxes_not_zero_angle[:, 1] = bboxes_not_zero_angle[:, 1] * scale_y
        bboxes_not_zero_angle[:, 0] = bboxes_not_zero_angle[:, 0] * scale_x

    if bboxes_zero_angle.shape[0] > 0:
        bboxes_zero_angle[:, 3] = bboxes_zero_angle[:, 3] * scale_factor_y
        bboxes_zero_angle[:, 2] = bboxes_zero_angle[:, 2] * scale_factor_x
        bboxes_zero_angle[:, 1] = bboxes_zero_angle[:, 1] * scale_factor_y
        bboxes_zero_angle[:, 0] = bboxes_zero_angle[:, 0] * scale_factor_x

    return np.concatenate([bboxes_zero_angle, bboxes_not_zero_angle])


from timeit import default_timer as timer

bbox = [0, 0, 100, 100, 0]

t = timer()
for _ in range(100):
    res = scale_rbbox(np.array([bbox]), 2, 3)

print(f"Time to scale (python): {timer() - t}")

bbox = BBox(0, 0, 100, 100, None)

t = timer()
for _ in range(100):
    res = bbox.scale(2, 3)

print(f"Time to scale (rust): {timer() - t}")


from savant_rs.utils import gen_frame, save_message, load_message
from savant_rs.primitives import Message, ParentObject, Object, PolygonalArea, Point, BBox, Value, Attribute, VideoFrame, PyVideoFrameContent, PyFrameTransformation
import json
from timeit import default_timer as timer

f = gen_frame()
t = timer()
for _ in range(1_000):
    r = f.json
print(timer() - t)

r = json.loads(f.json)
print(type(r) is dict)

frame = VideoFrame(
    source_id="Test",
    framerate="30/1",
    width=1920,
    height=1080,
    content=PyVideoFrameContent.external("s3", "s3://some-bucket/some-key.jpeg"),
    codec="jpeg",
    keyframe=True,
    pts=0,
    dts=None,
    duration=None,
)

print(frame.json)

frame.width = 3840
frame.height = 2160
frame.dts = 100
frame.duration = 100

print(frame.json)

frame.add_transformation(PyFrameTransformation.initial_size(1920, 1080))
frame.add_transformation(PyFrameTransformation.scale(3840, 2160))
frame.add_transformation(PyFrameTransformation.padding(left=0, top=120, right=0, bottom=0))

print(frame.transformations)

print(frame.transformations[0].is_initial_size)
print(frame.transformations[0].as_initial_size)

print(frame.transformations[0].is_scale)

frame.clear_transformations()

frame.set_attribute(Attribute(creator="some", name="attr", hint="x", values=[
    Value.none(),
    Value.bytes(dims=[8, 3, 8, 8], blob=bytes(3 * 8 * 8), confidence=None),
    Value.integer(1, confidence=0.5),
    Value.float(1.0, confidence=0.5),
    Value.string("hello", confidence=0.5),
    Value.boolean(True, confidence=0.5),
    Value.strings(["hello", "world"], confidence=0.5),
    Value.integers([1, 2, 3], confidence=0.5),
    Value.floats([1.0, 2.0, 3.0], confidence=0.5),
    Value.booleans([True, False, True], confidence=0.5),
    Value.bbox(BBox(0.1, 0.2, 0.3, 0.4), confidence=0.5),
    Value.bboxes([BBox(0.1, 0.2, 0.3, 0.4), BBox(0.1, 0.2, 0.3, 0.4)], confidence=0.5),
    Value.point(Point(0.1, 0.2), confidence=0.5),
    Value.points([Point(0.1, 0.2), Point(0.1, 0.2)], confidence=0.5),
    Value.polygon(PolygonalArea([Point(-1, 1), Point(1, 1), Point(1, -1), Point(-1, -1)], ["up", None, "down", None]), confidence=0.5),
    Value.polygons([
        PolygonalArea([Point(-1, 1), Point(1, 1), Point(1, -1), Point(-1, -1)], ["up", None, "down", None]),
        PolygonalArea([Point(-1, 1), Point(1, 1), Point(1, -1), Point(-1, -1)], ["up", None, "down", None])], confidence=0.5),
]))

frame.set_attribute(Attribute(creator="other", name="attr", values=[
    Value.integer(1, confidence=0.5),
]))

print(frame.attributes)

print(frame.find_attributes(name="attr"))
print(frame.find_attributes(creator="other"))
print(frame.find_attributes(creator="other", name="attr"))
print(frame.find_attributes(hint="x"))
print(frame.find_attributes(creator="some", hint="x"))

print(frame.get_attribute(creator="other", name="attr"))
deleted = frame.delete_attribute(creator="some", name="attr")
print(deleted)

frame.add_object(Object(
    id=1,
    creator="some",
    label="person",
    bbox=BBox(0.1, 0.2, 0.3, 0.4),
    confidence=0.5,
    attributes={},
    parent=ParentObject(id=2, creator="some", label="car"),
    track_id=None,
))

o = frame.access_objects_by_id([1])
o = o[0]
print(o)

o.set_attribute(Attribute(creator="other", name="attr", values=[
    Value.integer(1, confidence=0.5),
]))

o.set_attribute(Attribute(creator="some", name="attr", values=[
    Value.integers([1, 2, 3], confidence=0.5),
]))

message = Message.video_frame(frame)

t = timer()

frame_message = None
for _ in range(1_000):
    bytes = save_message(message)
    frame_message = load_message(bytes)

print(timer() - t)

print(frame_message.is_video_frame)
frame = frame_message.as_video_frame

# print(frame)





import threading
from pprint import pprint
from numba import jit
from random import choice
from timeit import default_timer as timer


from savant_rs.utils import RegistrationPolicy, \
    build_model_object_key, \
    is_model_registered, \
    is_object_registered, \
    parse_compound_key, \
    validate_base_key, \
    clear_symbol_maps, \
    dump_registry, \
    get_model_id, \
    get_object_id, \
    get_object_ids, \
    get_model_name, \
    get_object_label, \
    get_object_labels, \
    register_model_objects


@jit(nopython=True, nogil=True)
def plus_one() -> int:
    return 1


cntr = 0
exit_flag = False

def threaded_count():
    global cntr, exit_flag
    while not exit_flag:
        cntr += plus_one()


models = ["model1", "model2", "model3", "model4", "model5", "model6", "model7", "model8", "model9", "model10"]

t = timer()
cnt = 0
while cnt < 10_000_000:
    cnt += plus_one()

mil_count = timer() - t
print(f"Time to count to million: {mil_count}")

clear_symbol_maps()

key = build_model_object_key("model1", "object_1_model1")
assert key == "model1.object_1_model1"

num = 0
for m in models:
    d = dict([(id, f"object_{id}_{m}") for id in range(1000)])
    t = timer()
    register_model_objects(m, d, RegistrationPolicy.ErrorIfNonUnique)
    num += timer() - t

print(f"Time to register: {num}")

model = get_model_name(1)
assert model == "model2"

assert is_model_registered("model2")
assert is_object_registered("model2", "object_1_model2")

m, l = parse_compound_key("model2.object_1_model2")
key = validate_base_key("model2")

label = get_object_label(1, 0)
assert label == "object_0_model2"


def simple_count():
    total_time = timer()
    random_model = choice(models)
    random_object = f"object_{choice(range(1000))}_{random_model}"
    num = 10_000_000
    for _ in range(num):
        m = get_model_id(model_name=random_model)
        m, o = get_object_id(model_name=random_model, object_label=random_object)
    return num, timer() - total_time

num, single_total_time = simple_count()

cntr = 0
exit_flag = False
t = threading.Thread(target=threaded_count)
t.start()
num, total = simple_count()
exit_flag = True
t.join()
print(f"Time to get in individually (total: {total}, idle: {total - single_total_time}): {num}, cnt: {cntr}, time spent in counter {cntr / 10_000_000 * mil_count}")


def batched_count():
    total_time = timer()
    random_model = choice(models)
    random_objects = [f"object_{choice(range(1000))}_{random_model}" for _ in range(100)]
    num = 100000
    for _ in range(num):
        get_object_ids(model_name=random_model, object_labels=random_objects)
    return num, timer() - total_time


num, single_total_time = batched_count()

cntr = 0
exit_flag = False
t = threading.Thread(target=threaded_count)
t.start()
num, total = batched_count()
exit_flag = True
t.join()
print(f"Time to get in bulk(total: {total}, idle: {total - single_total_time}): {num}, cnt: {cntr}, time spent in counter {cntr / 10_000_000 * mil_count}")

ids = get_object_ids(model_name="model1", object_labels=["object_1_model1", "object_2_model1", "object_X_model1"])
assert ids == [('object_1_model1', 1), ('object_2_model1', 2), ('object_X_model1', 1000)]

labels = get_object_labels(model_id=0, object_ids=[1, 2, 100000])
assert labels == [(1, 'object_1_model1'), (2, 'object_2_model1'), (100000, None)]

# pprint(dump_registry())



"""FPS measurement."""
from time import time
from typing import Optional


# pylint:disable=too-many-instance-attributes
class FPSMeter:
    """FPS measurement.

    :param period_frames: FPS measurement period, in frames
    :param period_seconds: FPS measurement period, in seconds
    """

    def __init__(
        self,
        period_frames: Optional[int] = None,
        period_seconds: Optional[float] = None,
    ):
        self._period_frames = 100000
        self._period_seconds = None

        if period_frames is not None:
            assert (
                period_seconds is None
            ), 'Only one of "period_frames" or "period_seconds" should be set'
            self.period_frames = period_frames
        elif period_seconds is not None:
            self.period_seconds = period_seconds

        self._frame_counter = 0
        self._last_frame_counter = 0
        self._start_time = 0.0
        self._last_start_time = 0.0

    @property
    def period_frames(self):
        """FPS measurement period, in frames."""
        return self._period_frames

    @period_frames.setter
    def period_frames(self, value: int):
        """FPS measurement period, in frames."""
        assert value > 0
        self._period_frames = value
        self._period_seconds = None

    @property
    def period_seconds(self):
        """FPS measurement period, in seconds."""
        return self._period_seconds

    @period_seconds.setter
    def period_seconds(self, value: float):
        """FPS measurement period, in seconds."""
        assert value > 0
        self._period_frames = None
        self._period_seconds = value

    @property
    def frame_counter(self):
        """Frame number."""
        return self._last_frame_counter

    @property
    def exec_seconds(self):
        """Execution time, sec."""
        return time() - self._last_start_time

    @property
    def fps(self):
        """FPS."""
        return self.frame_counter / self.exec_seconds

    @property
    def message(self):
        """Default FPS measurement message."""
        frame_counter = self.frame_counter
        frame_word = 'frame' if frame_counter == 1 else 'frames'
        return f'Processed {frame_counter} {frame_word}, {self.fps:.2f} FPS.'

    def start(self):
        """Starts the meter."""
        self._frame_counter = 0
        self._last_frame_counter = 0
        self._start_time = time()
        self._last_start_time = 0.0

    def __call__(self, frame_count: int = 1) -> bool:
        """
        :return: True if FPS measurement has occurred
        """
        # first call: init
        if not self._start_time:
            self._start_time = time()
            return False

        # next call: increment
        self._frame_counter += frame_count
        self._last_frame_counter = self._frame_counter
        self._last_start_time = self._start_time

        # reset after period
        if self._period_passed():
            self.reset_counter()
            return True

        return False

    def reset_counter(self):
        """Reset frame counter."""
        self._frame_counter = 0
        self._start_time = time()

    def _period_passed(self):
        if self._period_seconds:
            return self.exec_seconds >= self._period_seconds

        return self._frame_counter >= self._period_frames


from timeit import default_timer as timer

m = FPSMeter(period_seconds=5)
m.start()

t = timer()

for i in range(10_000_000):
    m(i)

print(m.message)
print(timer() - t)


from timeit import default_timer as timer


class Internal:
    def __init__(self, val):
        self.val = val

    def inc(self):
        self.val += 1


class Wrapper:
    def __init__(self, val):
        self.val = Internal(val)

    def get(self):
        return self.val


w = Wrapper(1)
t = timer()

for _ in range(1_000_000):
    i = w.get()
    i.inc()

print(timer() - t)
i = w.get()
print(i.val)
